# Copyright 2012, 2013 Marco Ceppi, Canonical Ltd.  This software is
# licensed under the GNU Affero General Public License version 3 (see
# the file LICENSE).

import logging
import urllib
import json

from charmworld.models import getconnection
from charmworld.models import getdb
from charmworld.utils import (
    configure_logging,
    get_ini,
)
from config import CHARM_IMPORT_FILTER
from config import CHARM_QUEUE
from utils import lock
from utils import LockHeld
from utils import get_queue
from utils import parse_branch

BRANCH_TIPS = "https://api.launchpad.net/devel/charm?ws.op=getBranchTips"


def available_charms(charm_data=None, limit=None):
    log = logging.getLogger("charm.launchpad")
    if not charm_data:
        charm_data = urllib.urlopen(BRANCH_TIPS).read()
        charm_data = json.loads(charm_data)

    count = 0

    for repo, commit, series in charm_data:
        _, branch_name = repo.rsplit("/", 1)

        try:
            data = parse_branch(repo, series, commit)
        except ValueError:
            log.warning("Unable to parse repo %s", repo)
            continue

        if CHARM_IMPORT_FILTER and not data['branch_spec'].startswith(
                CHARM_IMPORT_FILTER):
            continue

        if data["bname"] not in ("trunk", "trunk-1"):
            log.debug("Skipped branch %s", repo)
            continue

        log.info("Queueing %s", data['branch_spec'])
        yield data

        count += 1

        if limit and count >= limit:
            log.info("Import limit reached (%d)... stopping", limit)
            break


def queue_charms(out_queue, limit=None):
    log = logging.getLogger("charm.launchpad")
    for charm in available_charms(limit=limit):
        added = out_queue.put(charm)
        if added and 0:
            log.info("Queued %s", charm)


if __name__ == '__main__':
    configure_logging()
    log = logging.getLogger("charm.launchpad")
    settings = get_ini()
    charm_import_limit = settings.get('charm_import_limit')
    charm_import_limit = (
        int(charm_import_limit) if charm_import_limit else None)
    connection = getconnection(settings)
    db = getdb(connection, settings.get('mongo.database'))
    try:
        with lock('ingest-queue', int(settings['script_lease_time']) * 60,
                  db, log):
            queue = get_queue(CHARM_QUEUE)
            queue_charms(queue, charm_import_limit)
    except LockHeld, error:
        log.warn(str(error))
