# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sys

from charmworld.jobs.ingest import ProofIngestJob
from charmworld.testing import JobTestBase


class ProofTest(JobTestBase):

    def test_charm_proof_library_not_importable(self):
        # If the charm proof library cannot be imported, an error is logged
        # and the proof loop is aborted.
        job = ProofIngestJob()
        handler = self.get_handler('charm.proof')
        not_importable_path = sys.path[0]
        job.setup(proof_path=not_importable_path)
        log_messages = [record.getMessage() for record in handler.buffer]
        err = ("proof error before processing began: "
               "could not import charm proof lib.")
        self.assertIn(err, log_messages)
        self.assertIn(
            'CHARM_PROOF_PATH: %s' % not_importable_path, log_messages)

    def test_charm_proof_library_not_found(self):
        # If the charm-proof library cannot be found, an error is logged
        # and the proof loop aborted.
        nonexistant_path = '/does/not/exist/'
        job = ProofIngestJob()
        handler = self.get_handler('charm.proof')
        job.setup(proof_path=nonexistant_path)
        log_messages = [record.getMessage() for record in handler.buffer]
        err = ("proof error before processing began: "
               "could not find charm proof path.")
        self.assertIn(err, log_messages)
        self.assertIn('CHARM_PROOF_PATH: %s' % nonexistant_path, log_messages)
