# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from charmworld.jobs.ingest import BzrIngestJob
from charmworld.jobs.ingest import ScanIngestJob
from charmworld.testing import factory
from charmworld.testing import JobTestBase
from charmworld.testing import REAL_FILE_CHARM_DIR
from charmworld.utils import quote_key


class TestScanJob(JobTestBase):

    def test_config_metadata_with_dotted_keys(self):
        bzr = BzrIngestJob()
        scan = ScanIngestJob()
        bzr.setup(db=self.db, root_dir=REAL_FILE_CHARM_DIR)
        scan.setup(db=self.db)
        charm_id, charm = factory.makeCharm(self.db, with_real_files=True)
        del charm['config']
        charm['files'] = dict([
            (quote_key(cfile.filename), dict(cfile)) for cfile in
            bzr.store_branch_files(charm)
        ])
        scan.run(charm)
        stored_data = self.db.charms.find_one({'_id': charm['branch_spec']})
        # The key 'dotted.name' is replaced with 'dotted%2Ename', and
        # 'foo.bar' is replaced with 'foo%2Ebar'.
        self.assertFalse('dotted.name' in stored_data)
        self.assertEqual('foo', stored_data['dotted%2Ename'])
        self.assertFalse('foo.bar' in stored_data['config']['options'])
        self.assertEqual('baz', stored_data['config']['options']['foo%2Ebar'])
