# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import json
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound
from webob import Response

from charmworld.search import ElasticSearchClient
from charmworld.views.helpers import result_sorter
from charmworld.utils import get_ini


def do_search(terms):
    return ElasticSearchClient.from_settings(get_ini()).search(terms)


@view_config(
    route_name="search",
    renderer="charmworld:templates/search.pt")
def search(request):
    if not "search_text" in request.params:
        url = request.route_url("home")
        return HTTPFound(location=url)
    search = do_search(request.params["search_text"])
    return dict((key, value) for key, value in search.items()
                if key != 'matches')


@view_config(
    route_name="search-json")
def search_json(request):
    data = do_search(request.params["search_text"])
    results = data['results']
    charms = []
    for r in results:
        charm = r['data']
        if not 'store_url' in charm:
            continue

        charms.append({
            'store_url': charm['store_url'],
            'name': charm['name'],
            'summary': charm['summary'],
            'series': charm['series'],
            'owner': charm['owner'],
            'data_url': '%s/json' % (
                # request.host_url,
                charm['short_url']),
            'relevance': r['weight'],
        })

        if 'subordinate' in data:
            charms[-1]['subordinate'] = True

    charms.sort(result_sorter)

    data.update({"results": charms,
                 "results_size": len(charms)})

    return Response(
        json.dumps(data, indent=2),
        headerlist=[
            ("Access-Control-Allow-Origin", "*"),
            ("Access-Control-Allow-Headers", "X-Requested-With")],
        content_type=u"application/json")
