# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import imp
from charmworld.testing import MongoTestBase
from charmworld.testing.factory import random_string


class TestMigration002(MongoTestBase):
    """Keys in charm data containing dots, dollar or perecent signs
    are changed to variants with containing escaped variants od these
    characters.
    """

    def test_upgrade(self):
        module = imp.load_source(
            'remove_dots',
            'migrations/versions/'
            '002_Escape_dots_dollar_signs_percent_signs_in_mongo_keys.py')


        charm_1 = {
            '_id': random_string(),
            'some_key': 'whatever',
        }
        charm_2 = {
            '_id': random_string(),
            'foo.bar': 'something else',
            'nested': {
                'a.b': 'something completely different',
            }
        }
        # A simple insert() call would fail: Even the PyMongo version used
        # now (2.0.1) checks if dot exist in keys. But update() does not
        # check the keys.
        self.db.charms.update({'_id': charm_1['_id']}, charm_1, upsert=True)
        self.db.charms.update({'_id': charm_2['_id']}, charm_2, upsert=True)
        module.upgrade(self.db)
        charm_1_upgraded = self.db.charms.find_one({'_id': charm_1['_id']})
        self.assertEqual(charm_1, charm_1_upgraded)
        charm_2_upgraded = self.db.charms.find_one({'_id': charm_2['_id']})
        self.assertEqual(
            {
                '_id': charm_2['_id'],
                'foo%2Ebar': 'something else',
                'nested': {
                    'a%2Eb': 'something completely different',
                }
            },
            charm_2_upgraded)
